#include <cstdint>
#include <cstring>
#include "janitza.h"
#include "common.h"

#define MODBUS_TRANSMISSION_DELAY 2

const char* JanitzaUnitStr(JanitzaUnit unit) {
    switch (unit) {
        case JanitzaUnit::V: return "V";
        case JanitzaUnit::A: return "A";
        case JanitzaUnit::W: return "W";
        case JanitzaUnit::VA: return "VA";
        case JanitzaUnit::Wh: return "Wh";
        default: return "[?]";
    }
}

void JanitzaReader::configure() {
#ifndef CONFIG_JANITZA_EMULATE
    pinMode(CONFIG_MODBUS_DIR_PIN, OUTPUT);
    digitalWrite(CONFIG_MODBUS_DIR_PIN, LOW);

    Serial2.begin(CONFIG_MODBUS_SERIAL_BAUD, SERIAL_8N1, CONFIG_MODBUS_RX_PIN, CONFIG_MODBUS_TX_PIN);
    Serial2.setTimeout(200);
    node.begin(CONFIG_MODBUS_SLAVE_ID, Serial2);

    node.preTransmission(JanitzaReader::modbusPreTransmissionCallback);
    node.postTransmission(JanitzaReader::modbusPostTransmissionCallback);
#endif
}

void JanitzaReader::modbusPreTransmissionCallback() {
#ifndef CONFIG_JANITZA_EMULATE
    delay(MODBUS_TRANSMISSION_DELAY);
    digitalWrite(CONFIG_MODBUS_DIR_PIN, HIGH);
#endif
}

void JanitzaReader::modbusPostTransmissionCallback() {
#ifndef CONFIG_JANITZA_EMULATE
    digitalWrite(CONFIG_MODBUS_DIR_PIN, LOW);
    delay(MODBUS_TRANSMISSION_DELAY);
#endif
}

bool JanitzaReader::readFloat(const JanitzaRegister& reg, float* val) {
#ifndef CONFIG_JANITZA_EMULATE
    uint8_t result;
    uint16_t buf[2];

    result = node.readInputRegisters(reg.addr, 2);
    if (result == node.ku8MBSuccess) {
        buf[1] = node.getResponseBuffer(0x00);
        buf[0] = node.getResponseBuffer(0x01);
        *val = *((float*)buf);
        PRINTF("JanitzaReader: %s = %.2f\n", reg->name, val);
        return true;
    } else {
        lastErrorCode = result;
        PRINTF("JanitzaReader: failed to read %d, device response: %d\n", reg.addr, result);
        return false;
    }
#else
#if CONFIG_JANITZA_EMULATE_ERROR == 1
    PRINTLN("JanitzaReader: emulation read error");
    return false;
#endif

    if (startsWith(reg.name, "_ULN") || startsWith(reg.name, "_ULN")) {
        *val = 220.0f;
    } else if (startsWith(reg.name, "_ILN")) {
        *val = 5.0f;
    } else if (startsWith(reg.name, "_I_SUM3")) {
        *val = 15.0f;
    } else if (startsWith(reg.name, "_PLN")) {
        *val = 100.0f;
    } else if (!strcmp(reg.name, "_P_SUM3")) {
        *val = 250.0f;
    } else if (startsWith(reg.name, "_SLN")) {
        *val = 400.0f;
    } else if (startsWith(reg.name, "_WH")) {
        *val = 450.0f;
    }
    return true;
#endif
}

uint8_t JanitzaReader::getLastErrorCode() const {
    return lastErrorCode;
}